#include "diff_eq_system.h"

void setDefectConfig(double* config, double* defect_config) {
    for (int i=0; i<13; i++) {
    defect_config[i] = config[i];
    }
}


//kartezicne koo.v sfericne
void setLattice(int l, double* spherical_lattice, double* lattice_thetas, double* lattice_phis) {
    double xy;
    for (int i=0; i<l; i++) {
    xy = sqrt(spherical_lattice[3*i] * spherical_lattice[3*i]
                + spherical_lattice[3*i+1] * spherical_lattice[3*i+1]);
    lattice_thetas[i] = atan2(xy, spherical_lattice[3*i+2]);
    lattice_phis[i] = atan2(spherical_lattice[3*i+1], spherical_lattice[3*i]);
    }
}


int count_sphere_points(double* center, double radius){
    double distance;
    int count_points = 0;
//     #pragma omp parallel for reduction (+:count_points)
    for(int l=0;l<NMAX;l++){
        
        float x,y,z,r; 
            
        x=(float)(i_vr(l))-(float)(I-1)/2+0.001;
        y=(float)(j_vr(l))-(float)(J-1)/2+0.001;
        z=(float)(k_vr(l))-(float)(K-1)/2+0.001;
        
        distance = (x - center[0]) * (x - center[0]) + (y - center[1]) * (y - center[1]) + (z - center[2]) * (z - center[2]);
        
        if(LMARK[l]&(LMARKPOV) && distance < 1.01 * radius * radius){count_points++;}
        
    }    
    return count_points;
}


void points_sphere(double* center, double radius, double* points, int* association){

    int k = 0;
    for(int l=0;l<NMAX;l++){
        
        if(LMARK[l]&(LMARKPOV)){
            
            float x,y,z,r; 
            
            x=(float)(i_vr(l))-(float)(I-1)/2+0.001;
            y=(float)(j_vr(l))-(float)(J-1)/2+0.001;
            z=(float)(k_vr(l))-(float)(K-1)/2+0.001;
            
            if ((x - center[0]) * (x - center[0]) + (y - center[1]) * (y - center[1]) + (z - center[2]) * (z - center[2]) < 1.01 * radius * radius) {
//                 LMARK[l]=LMARKSPH;
                
                points[3*k]=x;
                points[3*k+1]=y;
                points[3*k+2]=z;
                association[k] = l;
                k++;
                
                
            }
        
    }
    }
    
}



void active_droplet_var(ActiveDropletVariables* active_variables){
//     active_variables->
    active_variables->active_nu = 2;
    active_variables->active_tau = 2;
    active_variables->drag_mu = 2;
    active_variables->defect_size = 0.001;
    active_variables->xi_rot = 4 * active_variables->active_tau / log(1 / active_variables->defect_size);
    active_variables->xi_alpha = active_variables->xi_rot;
    active_variables->active_chi = 2 / log(1 / active_variables->defect_size);    
    active_variables->active_dt = dt_active0 * dt_active0 * active_variables->active_nu * R / VELOCITY_SCALING;  // timestep for active simulation for defect velocity to mach rotation time
    
}

// everything on a sphere
void init_droplet(ActiveDroplet* sphere) {
    
    // double center1[3] = {0., 0., 0.};
    double original_config[13] = {2.2549091,
                                    2.5599512,
                                    0.8845974,
                                    0.5845573,
                                    0.9355829,
                                    3.9472116,
                                    4.7124571,
                                    1.7150761,
                                    0.8025151,
                                    0.6296737,
                                    -2.3309994,
                                    -2.5228739,
                                    2.6938891};
    
    
    int num_points = count_sphere_points(sphere->center, sphere->radius);
    double* all_points = (double *)malloc(3 * num_points*sizeof(double));
    int* assoc = (int *)malloc(num_points*sizeof(int));
    points_sphere(sphere->center, sphere->radius, all_points, assoc);

    sphere->num_points = num_points;
    sphere->association = (int*)malloc(num_points*sizeof(int));
    for (int i=0;i<num_points; i++) {
        sphere->association[i]=assoc[i];
//         if (i<100) {
//             printf("%d\n", sphere->association[i]);
//         }
    }
    sphere->lattice_thetas = (double *)malloc(num_points*sizeof(double));
    sphere->lattice_phis = (double *)malloc(num_points*sizeof(double));
    sphere->velocities = (double *)malloc(3*num_points*sizeof(double));
                                
    setLattice(sphere->num_points, all_points, sphere->lattice_thetas, sphere->lattice_phis);
    setDefectConfig(original_config, sphere->defect_config);
    
    sphere->lattice_basis = (double *)malloc(num_points*9*sizeof(double));
    lattice_base(sphere->num_points, sphere->lattice_thetas, sphere->lattice_phis, sphere->lattice_basis);
    
    sphere->theta_grad = (double *)malloc(sphere->num_points * sizeof(double));
    sphere->phi_grad = (double *)malloc(sphere->num_points * sizeof(double));

}

void velocity_grad(ActiveDroplet* sphere){

    for(int l=0;l<NMAX;l++){
        
        gradU[1][l]=0;
        gradU[2][l]=0;
        gradU[3][l]=0;
    }
     
    double grad_x;
    double grad_y;
    double grad_z;
    double vec_theta[3];
    double vec_phi[3];
    
    #pragma omp parallel for num_threads(STPROC) schedule(dynamic, 2500)
    for(int k=0;k<sphere->num_points;k++){
        float ux[4],uy[4],uz[4];
        float n1,n2,n3;
        

        float pos[4];
        pos[1]=(float)(i_vr(sphere->association[k])); //x
        pos[2]=(float)(j_vr(sphere->association[k])); //y
        pos[3]=(float)(k_vr(sphere->association[k])); //z
        float n[4];
        n[1]=QNIC[4][sphere->association[k]];
        n[2]=QNIC[5][sphere->association[k]];
        n[3]=QNIC[6][sphere->association[k]];
        
        float grad[4];
        
        
        izrOdvU2smeri(pos, n, grad);
        

        gradU[1][sphere->association[k]]=grad[1];
        gradU[2][sphere->association[k]]=grad[2];
        gradU[3][sphere->association[k]]=grad[3];
        
        etheta(sphere->lattice_phis[k], sphere->lattice_thetas[k], vec_theta);
        ephi(sphere->lattice_phis[k], vec_phi);
        
        
        // Calculated gradient has direction opposite to flow velocity. In propagation code it should be taken in the direction of flow, so we add a minus.
        // Gradients are also scaled with the velocity_ratio factor and DT to transform magnitudes to appropriate units.
        sphere->theta_grad[k] = -(grad[1]*vec_theta[0] + grad[2]*vec_theta[1] + grad[3]*vec_theta[2])/VELOCITY_SCALING*DT;
        sphere->phi_grad[k] = -(grad[1]*vec_phi[0] + grad[2]*vec_phi[1] + grad[3]*vec_phi[2])/VELOCITY_SCALING*DT;      
        
//         fprintf(stderr, "gradient %f\n",grad[1]);
//         fprintf(stderr, "lokacija %d\n",(int)(pos[1]));
//         fprintf(stderr,"%f\n", (grad[1]*n[1]+grad[2]*n[2]+grad[3]*n[3])/sqrt(grad[1]*grad[1]+grad[2]*grad[2]+grad[3]*grad[3]));
        
        

    };

}

void defect_grad(ActiveDroplet* sphere){

    double vec_theta[3];
    double vec_phi[3];

    float pos_defect[4];
    FILE *defect_con;
    FILE *defect_all_grad;

    for(int j=0;j<4;j++){
        double loc_defect[3];
        double r=sphere->radius;
        
        sph_to_cart(sphere->defect_config[j], sphere->defect_config[j+4], loc_defect);            

        pos_defect[1]=loc_defect[0]*r + (float)(I-1)/2+0.001; //x
        pos_defect[2]=loc_defect[1]*r + (float)(J-1)/2+0.001; //y
        pos_defect[3]=loc_defect[2]*r + (float)(K-1)/2+0.001; //z
   
        float n[4];
        n[1]=loc_defect[0];
        n[2]=loc_defect[1];
        n[3]=loc_defect[2];
        
        float grad_defect[4];
        izrOdvU2smeri(pos_defect, n, grad_defect);
        
        
        etheta(sphere->defect_config[j+4], sphere->defect_config[j], vec_theta);
        ephi(sphere->defect_config[j+4], vec_phi);
        
        // Calculated gradient has direction opposite to flow velocity. In velocity field code it should be taken in the direction of flow, so we add a minus.
        // Gradients are also scaled with the velocity_ratio factor and DT to transform magnitudes to appropriate units.
        sphere->defect_theta_grad[j] = -(grad_defect[1]*vec_theta[0] + grad_defect[2]*vec_theta[1] + grad_defect[3]*vec_theta[2])/VELOCITY_SCALING*DT;
        sphere->defect_phi_grad[j] = -(grad_defect[1]*vec_phi[0] + grad_defect[2]*vec_phi[1] + grad_defect[3]*vec_phi[2])/VELOCITY_SCALING*DT;     
        
        sphere->defect_gradient[j] = sqrt(grad_defect[1]*grad_defect[1]+grad_defect[2]*grad_defect[2]+grad_defect[3]*grad_defect[3])/VELOCITY_SCALING*DT;

    }

}

void zapis_all_grad(ActiveDroplet* sphere){

    float pos_defect[4];
    FILE *defect_con;
    FILE *defect_all_grad;

    float all_grad_defect[12];
    
    for(int j=0;j<4;j++){
        double loc_defect[3];
        double r=sphere->radius;
        
        sph_to_cart(sphere->defect_config[j], sphere->defect_config[j+4], loc_defect);            
        
        pos_defect[1]=loc_defect[0]*r + (float)(I-1)/2+0.001; //x
        pos_defect[2]=loc_defect[1]*r + (float)(J-1)/2+0.001; //y
        pos_defect[3]=loc_defect[2]*r + (float)(K-1)/2+0.001; //z    
        
        float n[4];
        n[1]=loc_defect[0];
        n[2]=loc_defect[1];
        n[3]=loc_defect[2];
        
        float grad_defect[4];
        
        
        izrOdvU2smeri(pos_defect, n, grad_defect);
        
        all_grad_defect[3*j] = grad_defect[1]/VELOCITY_SCALING*DT;
        all_grad_defect[3*j+1] = grad_defect[2]/VELOCITY_SCALING*DT;
        all_grad_defect[3*j+2] = grad_defect[3]/VELOCITY_SCALING*DT;

        
    }
    
    #if TWRITEgrad
      //         zapis defect_config;
        defect_con=fopen("./data/defect_config.txt","a");
        
        if(defect_con==NULL){
            fprintf(stderr, "Failed opening file. a mode. file=%s\n", "./defect_con.txt");
            exit(1); 
        }
        for(int c=0;c<13;c++){
            fprintf(defect_con, "%lf;",sphere->defect_config[c]);
        };
        
        fprintf(defect_con, "\n");
        fclose(defect_con);
        
      //         zapis defect_all_grad;  
        defect_all_grad=fopen("./data/defect_all_grad.txt","a");
        if(defect_all_grad==NULL){
            fprintf(stderr, "Failed opening file. a mode. file=%s\n", "./defect_all_grad.txt");
            exit(1); 
        }
        for(int c=0;c<12;c++){
            fprintf(defect_all_grad, "%lf;",all_grad_defect[c]);
        };
        
//         fprintf(defect_all_grad, "%f;",all_grad_defect);
        fprintf(defect_all_grad, "\n");
        fclose(defect_all_grad);

       
    #endif

}

void velocity_sphere(ActiveDroplet* sphere, ActiveDropletVariables* active_variables){
//     propagate_trajectories_block(sphere->defect_config, nu, chi, xi_rot, xi_alpha, dt, num_steps);
//     double active_nu = 1.5;
//     double active_tau = 2;
//     double drag_mu = 2;
//     double defect_size = 0.001;
//     double xi_rot = 4 * active_tau / log(1 / defect_size);
//     double xi_alpha = xi_rot;
//     double active_chi = 2 / log(1 / defect_size);
//     
//     double active_dt = dt_active0*dt_active0*active_nu*R/VELOCITY_SCALING;  // timestep for active simulation for defect velocity to mach rotation time
//     
//     printf("REAL xi_rot = %f\n", xi_rot);
 
    
    double gradient=0;
    for(int j=0;j<4;j++){gradient+=sphere->defect_gradient[j];}
    
    double active_nu_eff = active_variables->active_nu - active_variables->drag_mu * gradient/4;
    
//     fprintf(stderr, "active_nu_eff %f\n",active_nu_eff);
    defect_grad(sphere);
    
    propagate_trajectories_step_drag(sphere->defect_config, sphere->defect_theta_grad, sphere->defect_phi_grad,
                                     active_variables->active_nu, active_variables->active_chi, active_variables->drag_mu, 
                                     active_variables->xi_rot, active_variables->xi_alpha, active_variables->active_dt);
 
        
    
    // propagate_trajectories_block(sphere->defect_config, active_nu, active_chi, xi_rot, xi_alpha, dt, num_steps);
    
    velocity_grad(sphere);
    
    velocity_field_drag(sphere->defect_config, sphere->num_points, sphere->lattice_thetas, sphere->lattice_phis, 
                        sphere->lattice_basis, sphere->theta_grad, sphere->phi_grad, 
                        active_variables->active_nu, active_variables->drag_mu, 0.1, sphere->velocities, STPROC); //rho=0.1
    
//         velocity_field_fast(sphere->defect_config, sphere->num_points, sphere->lattice_thetas, sphere->lattice_phis,
//                             sphere->lattice_basis, active_nu, 0.1, sphere->velocities, STPROC); //rho=0.1
                            
    // velocity_field(sphere->defect_config, sphere->num_points, sphere->lattice_thetas, sphere->lattice_phis,active_nu, 0.1, sphere->velocities); 

    
//     x_to_z_transf(sphere->num_points, sphere->velocities);
    
//     #pragma omp parallel for num_threads(STPROC) schedule(dynamic, 2500)
    for(int k=0;k<sphere->num_points;k++){   
            float n1=QNIC[4][sphere->association[k]];	//Surface normal
            float n2=QNIC[5][sphere->association[k]];
            float n3=QNIC[6][sphere->association[k]];

            UNIC[1][sphere->association[k]]=VELOCITY_SCALING*sphere->velocities[3*k];		//surface velocity
            UNIC[2][sphere->association[k]]=VELOCITY_SCALING*sphere->velocities[3*k+1];
            UNIC[3][sphere->association[k]]=VELOCITY_SCALING*sphere->velocities[3*k+2];
            
            
//             if (k<1000) {printf("%f\n", sphere->velocities[3*k]*n1+sphere->velocities[3*k+1]*n2+sphere->velocities[3*k+2]*n3);}

//             if (k==1000) {printf("%f ", sphere->velocities[3*k]*n1+sphere->velocities[3*k+1]*n2+sphere->velocities[3*k+2]*n3);}
            


    }
    
    
}


//Writes the velocity GRADIENT (with Paraview header)
void zapisUgrad(char *ime){
  // do it in binary
  int l;
  float grad[3];
  FILE *file;

  file=fopen(ime,"w");
  if(file==NULL){
    fprintf(stderr, "Failed opening file. w mode. file=%s\n", ime);
     exit(1);
  }

  
  fprintf(file,"# vtk DataFile Version 3.0\nKnot\nBINARY\nDATASET STRUCTURED_POINTS\n");
  fprintf(file,"DIMENSIONS %d %d %d\n",I,J,K);
  fprintf(file,"ORIGIN 0 0 0\n");
  fprintf(file,"SPACING 1 1 1\n");
  fprintf(file,"POINT_DATA %d\n",I*J*K);
  fprintf(file,"VECTORS gradU float\n");
  
  
  for(l=0; l<NMAX; l++){
    // go to bigendian
    grad[0] = FloatSwap(gradU[1][l]); 
    grad[1] = FloatSwap(gradU[2][l]);
    grad[2] = FloatSwap(gradU[3][l]);
      
    
    fwrite(grad,sizeof(float),3,file);
  };
  fclose(file);
}

//Writes the velocity GRADIENT - DEFECT (with Paraview header)
void zapisUgradDEFECT(char *ime){
  // do it in binary
  int l;
  float grad_defect[3];
  FILE *file;

  file=fopen(ime,"w");
  if(file==NULL){
    fprintf(stderr, "Failed opening file. w mode. file=%s\n", ime);
     exit(1);
  }

  
  fprintf(file,"# vtk DataFile Version 3.0\nKnot\nBINARY\nDATASET STRUCTURED_POINTS\n");
  fprintf(file,"DIMENSIONS %d %d %d\n",I,J,K);
  fprintf(file,"ORIGIN 0 0 0\n");
  fprintf(file,"SPACING 1 1 1\n");
  fprintf(file,"POINT_DATA %d\n",I*J*K);
  fprintf(file,"VECTORS gradUdefect float\n");
  

  for(l=0; l<NMAX; l++){
    // go to bigendian
    grad_defect[0] = FloatSwap(gradU_defect[1][l]); 
    grad_defect[1] = FloatSwap(gradU_defect[2][l]);
    grad_defect[2] = FloatSwap(gradU_defect[3][l]);
      
    
    fwrite(grad_defect,sizeof(float),3,file);
  };
  fclose(file);
}



void locDefect(ActiveDroplet* sphere, char *ime){
    FILE *file;
    
    file=fopen(ime,"w");
    if(file==NULL){
        fprintf(stderr, "Failed opening file. w mode. file=%s\n", ime);
        exit(1);
    }
    
    fprintf(file,"x,y,z\n");
    //4 defects
    for(int i=0;i<4;i++){
        double loc_defect[3];
        double r=sphere->radius;
        
        sph_to_cart(sphere->defect_config[i], sphere->defect_config[i+4], loc_defect);    
        fprintf(file,"%f,%f,%f\n",loc_defect[0]*r + sphere->center[0], loc_defect[1]*r + sphere->center[1], loc_defect[2]*r + sphere->center[2]);
    }
    fclose(file);
    
    
}


//----------------------------------------FIBONACCI------------------------------------------------------------------------

void fibonacci_init(FibonacciLattice* fibonacci_lattice, int num_points, double center[3], double radius){
    double golden_ratio = 1.618033988749894;
    fibonacci_lattice->center = center;
    fibonacci_lattice->radius = radius;
    fibonacci_lattice->num_points = num_points;
    
    fibonacci_lattice->lattice_thetas = (double *)malloc(num_points*sizeof(double));
    fibonacci_lattice->lattice_phis = (double *)malloc(num_points*sizeof(double));
    
    for(int i=0;i<num_points;i++){
        fibonacci_lattice->lattice_phis[i] = (i+1)*(2-golden_ratio)*2*M_PI;
        fibonacci_lattice->lattice_thetas[i] = acos(2*(i+1)/(num_points+1)-1);
        
    }
    
    fibonacci_lattice->velocities = (double *)malloc(3*num_points*sizeof(double));
    
    fibonacci_lattice->lattice_basis = (double *)malloc(num_points*9*sizeof(double));
    lattice_base(fibonacci_lattice->num_points, fibonacci_lattice->lattice_thetas, fibonacci_lattice->lattice_phis, fibonacci_lattice->lattice_basis);
    
    fibonacci_lattice->theta_grad = (double *)malloc(fibonacci_lattice->num_points * sizeof(double));
    fibonacci_lattice->phi_grad = (double *)malloc(fibonacci_lattice->num_points * sizeof(double));
}


void grad_fibonacci(FibonacciLattice* fibonacci_lattice){
    double vec_theta[3];
    double vec_phi[3];
    float position[4];
    FILE *defect_con;
    FILE *defect_all_grad;
/*    
    for(int l=0;l<NMAX;l++){
        
        gradU_defect[1][l]=0;
        gradU_defect[2][l]=0;
        gradU_defect[3][l]=0;
    }*/
    
    for(int j=0;j<fibonacci_lattice->num_points;j++){
        
        double location[3]; //normirane kartezicne koordinate
        double r=fibonacci_lattice->radius;
        
        sph_to_cart(fibonacci_lattice->lattice_thetas[j], fibonacci_lattice->lattice_phis[j], location);            
        
        position[1]=location[0]*r + (float)(I-1)/2+0.001; //x
        position[2]=location[1]*r + (float)(J-1)/2+0.001; //y
        position[3]=location[2]*r + (float)(K-1)/2+0.001; //z
/*        fprintf(stderr, "position = %f, %f, %f;   ",position[1],position[2],position[3]);
        fprintf(stderr, "%f\n ", position[1]*position[1]+position[2]*position[2]+position[3]*position[3]);
 */      
        float n[4];
        n[1]=location[0];
        n[2]=location[1];
        n[3]=location[2];
        
        float grad_point[4];
        izrOdvU2smeri(position, n, grad_point);

        
        for(int i=0;i<3;i++){
            vec_theta[i] = fibonacci_lattice->lattice_basis[9*j+3+i];
            vec_phi[i] = fibonacci_lattice->lattice_basis[9*j+6+i];
        }
        

        
        // Calculated gradient has direction opposite to flow velocity. In velocity field code it should be taken in the direction of flow, so we add a minus.
        // Gradients are also scaled with the velocity_ratio factor and DT to transform magnitudes to appropriate units.
        fibonacci_lattice->theta_grad[j] = -(grad_point[1]*vec_theta[0] + grad_point[2]*vec_theta[1] + grad_point[3]*vec_theta[2])/VELOCITY_SCALING*DT;
        fibonacci_lattice->phi_grad[j] = -(grad_point[1]*vec_phi[0] + grad_point[2]*vec_phi[1] + grad_point[3]*vec_phi[2])/VELOCITY_SCALING*DT;     
           
    }

}



void velocity_fibonacci(ActiveDroplet* sphere, FibonacciLattice* fibonacci_lattice, ActiveDropletVariables* active_variables){
    
    grad_fibonacci(fibonacci_lattice);

    
    velocity_field_drag(sphere->defect_config, fibonacci_lattice->num_points, fibonacci_lattice->lattice_thetas, fibonacci_lattice->lattice_phis, 
                        fibonacci_lattice->lattice_basis, fibonacci_lattice->theta_grad, fibonacci_lattice->phi_grad, 
                        active_variables->active_nu, active_variables->drag_mu, 0.1, fibonacci_lattice->velocities, STPROC); //rho=0.1
}


//average fibonacci_lattice->velocities //vector 
void zapis_fibonacci_average_velocity(ActiveDroplet* sphere, FibonacciLattice* fibonacci_lattice, ActiveDropletVariables* active_variables){
    FILE *file;
    
    float Vx = 0;
    float Vy = 0;
    float Vz = 0;
    
//     fprintf(stderr,"active_nu = %f\n", active_variables->active_nu);
//     fprintf(stderr,"drag_mu = %f\n", active_variables->drag_mu);
//     fprintf(stderr,"active_tau = %f\n", active_variables->active_tau);
//     fprintf(stderr,"defect_size = %f\n", active_variables->defect_size);
//     fprintf(stderr,"xi_rot = %f\n", active_variables->xi_rot);
//     fprintf(stderr,"xi_alpha = %f\n", active_variables->xi_alpha);
//     fprintf(stderr,"active_chi = %f\n", active_variables->active_chi);
//     fprintf(stderr,"active_dt = %f\n", active_variables->active_dt);   
//  
   
    velocity_fibonacci(sphere, fibonacci_lattice, active_variables);
    
    for(int i=0;i<fibonacci_lattice->num_points;i++){
//         fprintf(stderr,"%f; ", fibonacci_lattice->velocities[3*i]);
//         fprintf(stderr,"Vx = %f\n", Vx);
        Vx = Vx + fibonacci_lattice->velocities[3*i];
        Vy = Vy + fibonacci_lattice->velocities[3*i+1];
        Vz = Vz + fibonacci_lattice->velocities[3*i+2];
        
    }
    /*
    fprintf(stderr,"%f,%f,%f\n", fibonacci_lattice->velocities[0],fibonacci_lattice->velocities[1],fibonacci_lattice->velocities[2]);
        */
/*    fprintf(stderr,"Vx = %f\n", Vx);
    fprintf(stderr,"%f,%f,%f\n", Vx/fibonacci_lattice->num_points,Vy/fibonacci_lattice->num_points, Vz/fibonacci_lattice->num_points);
 */       
    
    file=fopen("./data/fibonacci_velocity.txt","a");
        
    if(file==NULL){
        fprintf(stderr, "Failed opening file. a mode. file=%s\n", "./fibonacci_velocity.txt");
        exit(1); 
    }
    
    fprintf(file, "%f,%f,%f\n",Vx/fibonacci_lattice->num_points, Vy/fibonacci_lattice->num_points, Vz/fibonacci_lattice->num_points);
    
    fclose(file);
    
    
    
    
}
