//Logical markers for the datapoint characterisation
#define LMARK1			2				//bulk
#define LMARKKOL		4				//colloid
#define LMARKPOV		8				//surface node
#define LMARKRP			16				//Marker is a part of boundary conditions
#define LMARK0			32				//There is absolutely nothing to be done in that point


//Numerical parameters

//A time step in the code corresponds to RESP^2/L/GAM*DT, where RESP=resolution, L=elastic constant, GAM is GAMMA viscosity. All can be chosen arbitrarily
#define DT			0.025  //Changing DT does not alter material parameter values (but it affects the Reynolds number)

//relaxation time of the Lb scheme -- sets up the isotropic visosity alpha4. It is important for numerical precision, in particular the slip velocity at the walls.
#define TAUF			(2.f*DT)

//Density parameter - vaguely related to the actual density (note that LB is weakly compressible). Sets up the isotropic viscosity eta. The fact that the density is way too largre results in larger Reynolds number compared to experiments. eta=DENSITYINIT*DT*(TAUF/DT-0.5)/3 in units of 1/GAMMA. Lower DENSITYINIT might lead to numerical instabilities.
#define	DENSITYINIT		(2.75f/DT)



//Material parameters

//Alignment parameter
#define XI			1.f				//Tumbling/aligning regime -- transition is at XI=0.857 (at SEQ=0.533)

//Activity
float ZETA;//			0.0f				//Activity parameter. To get the actual value (in units of L/RESP^2) multiply by RESP^2/L

//Phase parameters (in the units of L/RESP^2)
#define A			(-0.43f)
#define B			(-5.3f)
#define C			(4.325f)
#define SEQ			(0.5*(-B/(3*C)+sqrt(fsqr(B/(3*C))-(8*A)/(3*C))) )
//For the above phase parameters, nematic correlation length KSI=sqrt(L/(A+B*Seq+9/2*C*Seq^2)) equals (0.663e-8) times RESP

// Surface anchoring (in the units of L/RESP)
#define WHOM			(0.5f)			//homeotropic anchoring strength
// #define WHOM			(0.005f)			//homeotropic anchoring strength


// Radius of a shell
#define R			(30.0f)			

//active droplet
#define dt_active0  (1.0E-4)  // ACTIVE DROPLET TIME/PASSIVE NEMATIC TIME  
#define VELOCITY_SCALING  (0.01)  


// 
// #define nu			  5
// #define rho			(0.1f)
// #define chi			1.0/log(1.0/rho)
// #define xi_rot			1.0
// #define tau_active			xi_rot / (2 * chi)
// #define xi_alpha			1.0




//A possible choice for the material parameters can correspond to the values that Miha Ravnik chose long time ago
// //parameters that define the scale of the system (not really, these are just a suggestion)
// #define RESP			(1e-8)				//Resolution (in meters)
// //#define KSI			(0.663e-8)
// #define L			(4.e-11)			//Elastic constant -- (in N)
// //Phase parameters
// #define SEQ			(0.5*(-B/(3*C)+sqrt(fsqr(B/(3*C))-(8*A)/(3*C))) )
// #define Ape			(-0.172e6)			// J/m3
// #define Bpe			(-2.12e6)			// J/m3
// #define Cpe			(1.73e6)			// J/m3
// #define A			(Ape*RESP*RESP/L)		// (for T-T*=-10K)   A=-0.43
// #define B			(Bpe*RESP*RESP/L)		// -5.3
// #define C			(Cpe*RESP*RESP/L)		// 4.325





//Pressure flow driving (typically not needed)
// #define DELTAP			0
// #define DELTAPX			(0.001f*DENSITYINIT)
// #define DELTAPY			(0.0f*DENSITYINIT)
